% Adaptive Mixed Grey Wolf Optimizer
%%% inspired by:

%%% the a parameter is polynomial
%___________________________________________________________________%
%  Grey Wold Optimizer (GWO) source codes version 1.0               %
%                                                                   %
%  Developed in MATLAB R2011b(7.13)                                 %
% %                                                                   %
%  Author and programmer: Seyedali Mirjalili                        %
%                                                                   %
%         e-Mail: ali.mirjalili@gmail.com                           %
%                 seyedali.mirjalili@griffithuni.edu.au             %
%                                                                   %
%       Homepage: http://www.alimirjalili.com                       %
%                                                                   %
%   Main paper: S. Mirjalili, S. M. Mirjalili, A. Lewis             %
%               Grey Wolf Optimizer, Advances in Engineering        %
%               Software , in press,                                %
%               DOI: 10.1016/j.advengsoft.2013.12.007               %
%                                                                   %
%___________________________________________________________________%


%%%% Mthodologie Discrete GWO:

%1) slection du leader soit alpha soit beta soit delta
%2) dplacement vers le leader de d units



function [Alpha_score,Alpha_pos,Convergence_curve,Final_Iter_Index]=GWO_aMixed_Fixed_Power_for_a...
    (nb_search_agent,Nb_Leaders_GWO,...
    it_max,power_for_a,...
    Parameter_Set_Search_Sets,Integer_Constrained,...
    Parameter_Set_Alpha_Initial_Position,min_FRR,dim,...
    Name_Objective_Function,Optimizer)

%%% power_for_a: \eta 

%%% Parameter_Set_Search_Sets is cell with vectors whose components are the admissible values for each expected parameter
%%% Integer_Constrained: vector containing the indices of the parameters
%%% which are strictly integer

%%% Parameter_Set_Alpha_Initial_Position position of alpha in terms of
%%% indices
Nb_Parameters=dim;


if isempty(Integer_Constrained)==0 %%% At least one parameter is discrete
    
    
    if length(Integer_Constrained)==Nb_Parameters %%% All parameters are discrete
        Discrete_Indices=[1:Nb_Parameters];
        Continuous_Indices=[];
    else %%% Most general case: there are both continuous and discrete indices
        Discrete_Indices=[1:max(Integer_Constrained)];
        Continuous_Indices=[1+max(Integer_Constrained):Nb_Parameters];
    end
    
    
elseif isempty(Integer_Constrained)==1%%% All parameters are continuous
    Discrete_Indices=[];
    Continuous_Indices=[1:Nb_Parameters];
    
end

%%% Redefinition of Parameter_Set_Alpha_Initial_Position, from guess value
%%% to guess index in the research sets.

Parameter_Set_Alpha_Initial_Position_Values=Parameter_Set_Alpha_Initial_Position;
Parameter_Set_Alpha_Initial_Position_Temp=zeros(size(Parameter_Set_Alpha_Initial_Position));
% tic

for j=1:Nb_Parameters
    [dummy,Parameter_Set_Alpha_Initial_Positionj]=min(abs(Parameter_Set_Alpha_Initial_Position_Values(j)-Parameter_Set_Search_Sets{j}));
    %%% index where the difference is min
    Parameter_Set_Alpha_Initial_Position_Temp(j)=Parameter_Set_Alpha_Initial_Positionj;
end

% Initialisation = toc
%
% pause on
% pause

Parameter_Set_Alpha_Initial_Position=Parameter_Set_Alpha_Initial_Position_Temp;


Parameter_Set_Lower_Bounds=zeros(1,dim);
Parameter_Set_Upper_Bounds=zeros(1,dim);

for ind_Parameter=1:dim
    Parameter_Set_Lower_Bounds(1,ind_Parameter)=Parameter_Set_Search_Sets{ind_Parameter,1}(1);
    Parameter_Set_Upper_Bounds(1,ind_Parameter)=Parameter_Set_Search_Sets{ind_Parameter,1}(end);
end



SearchAgents_no=nb_search_agent;
Max_iter=it_max;
lb=conj(Parameter_Set_Lower_Bounds(:)');
ub=conj(Parameter_Set_Upper_Bounds(:)');



% initialize alpha, beta, and delta_pos
Alpha_pos_Values=zeros(1,dim);
Alpha_pos_Indices=zeros(1,dim);


Alpha_score=inf; %change this to -inf for maximization problems

Beta_pos_Values=zeros(1,dim);
Beta_pos_Indices=zeros(1,dim);
Beta_score=inf; %change this to -inf for maximization problems

Delta_pos_Values=zeros(1,dim);
Delta_pos_Indices=zeros(1,dim);
Delta_score=inf; %change this to -inf for maximization problems

%Initialize the positions of search agents
[Positions_Indices,Positions_Values]=initialization_mixed(SearchAgents_no,dim,Parameter_Set_Search_Sets,ub,lb,...
    Parameter_Set_Alpha_Initial_Position,Parameter_Set_Alpha_Initial_Position_Values,Discrete_Indices,Continuous_Indices);


%%% Positions_Indices contains the indices in the search vectors for all wolves.
%Positions_Indices,Positions_Values
%Parameter_Set_Alpha_Initial_Position in terms of indices

%
% Minvalcont=repmat(lb(Continuous_Indices),SearchAgents_no,1);
% Maxvalcont=repmat(ub(Continuous_Indices),SearchAgents_no,1);
%
% if isempty(Continuous_Indices)==0
% Positions_Values_Continuous= Minvalcont + (Maxvalcont-Minvalcont).*rand(SearchAgents_no,length(Continuous_Indices));
% Positions_Values(1:SearchAgents_no,Continuous_Indices) = Positions_Values_Continuous;
% else
% Positions_Values=[];
% end


%%% Positions_Indices has size Nb Wolves \times Nb Parameters

%%% if Positions_Indices(i,j)=5 the wolf i, for the j^th parameter, bears the value which is in the component
%%% 5 in the vector containing the candidate values for the j^th parameter.

%%% Indices used in the programme:
%%% l: iteration
%%% i for wolf i;
%%% j for the parameter j (C may be for instance parameter 1; gamma may be parameter 2);
%%% ind_component for the location of the component in the vector containing the candidate values


Convergence_curve=zeros(1,Max_iter);


Set_Nb_Alpha_Chosen=zeros(1,Max_iter);
Set_Nb_Beta_Chosen=zeros(1,Max_iter);
Set_Nb_Delta_Chosen=zeros(1,Max_iter);
Set_a=zeros(1,Max_iter);

l=0;% Loop counter

%%%% Affichage fonction objectif
[X,Y,Z]=Compute_Values_For_Plot_Objective_Function(lb,ub,str2func(Name_Objective_Function));


Final_Iter_Index=Max_iter;% default value
% Main loop

%%% By default: we first look in discrete sets
%%% and every value in Set_Discrete_Values is 1;
%Set_Discrete_Values=zeros(1,Nb_Parameters);%%% 1 -> means discrete
%%% 0 -> means continuous

%Set_Discrete_Values(Integer_Constrained)=1;

Parameter_Set_Search_Sets_Discrete=cell(length(Discrete_Indices),1);
for indindices=Discrete_Indices
    Parameter_Set_Search_Sets_Discrete{indindices,1}=Parameter_Set_Search_Sets{indindices,1};
end


while l<Max_iter
    
    
    %
    %      Positions_Indices
    %     Positions_Values
    %l
    %
    
    % Alpha_score
    % Alpha_pos_Values
    
    %%% t_{exploration}=\frac{T}{2^{1/\eta}}
    %a=2*(1-l^power_for_a*(1/Max_iter^power_for_a)); % a decreases linearly from 2 to 0
    %Prop_Iterations_For_Discrete=1 / 2^(1/power_for_a);
    
    %Prop_Iterations_For_Pure_Discrete_Search=1.00;
    
    
    %power_for_a
    
    %Prop_Iterations_For_Discrete=0.9;
    
    
    %   if l>floor(Prop_Iterations_For_Pure_Discrete_Search*Max_iter)
    
    %
    % pause on
    % pause
    %
    
    %     Set_Discrete_Values=zeros(1,Nb_Parameters);
    %     Set_Discrete_Values(Integer_Constrained)=1;
    
    
    %     %%% The serach turns continous for the parameters
    %%% where this is possible
    %%% (example: SVM parameters C and gamma)
    
    %%% For all other parameters
    %%% the search remains discrete
    %%% because it has no sense to look for a value in a continous space
    %%% (example: rank value, k of K-NN classifier)
    %    end
    
    Nb_Alpha_Chosen=0;
    Nb_Beta_Chosen=0;
    Nb_Delta_Chosen=0;
    
    Time_Update_Leaders=0;
    %tic
    
    %for indextest=1:1
    %%% In total: 3.2 ms
    for i=1:nb_search_agent %%% For each wolf
        
        % Calculate objective function for each search agent
        %Positions(i,:)
        %         tic
        for j=Discrete_Indices%%% for all wolves and all parameters
            Positions_Values(i,j)=Parameter_Set_Search_Sets{j}(Positions_Indices(i,j));
        end
        %timetest00=toc   %%% 1 e-5
        
        
        %%% 0.1 ms * nb_search_agent
        %tic
        % Return back the search agents that go beyond the boundaries of the search space
        Flag4ub=Positions_Values(i,:)>ub;
        Flag4lb=Positions_Values(i,:)<lb;
        Positions_Values(i,:)=(Positions_Values(i,:).*(~(Flag4ub+Flag4lb)))+ub.*Flag4ub+lb.*Flag4lb;
        % timetest00=toc   %%% 1 e-5
        %            tic
        fitness=Run_Objective_Function(Name_Objective_Function,Positions_Values(i,:));%,Parameter_Set_imgs
        %%% Dominant in terms of computational load:
        %%%% 1 ms * nb_search_agent
        %timetest0=toc   %%% 2 e-2   dominant
        %
        %         pause on
        %         pause
        
        %%%%% Pour les fonctions Matlab Name_Objective_Function est:
        %%% Run_Matlab_Criterion_With_One_Parameter_Set
        %tic
        Update_Leaders %%% 0.6 ms * nb_search_agent
        %timetest=toc   %%% 7 e-5
        
        
    end
    %end
    %
    %     Time_Update_Leaders=toc   0.18  sec.
    %     pause on
    %     pause
    
    
    %time_Displacement=0;
    
    %    tic
    %    for indextest=1:1
    
    a=2*(1-(l+1)^power_for_a*(1/Max_iter^power_for_a));
    
    %     if l<ceil(2*Max_iter/4)
    %         a=2*(1-l^power_for_a*(1/Max_iter^power_for_a)); % a decreases linearly from 2 to 0
    %     else
    %         a=2*(1-l^(1/power_for_a)*(1/Max_iter^(1/power_for_a))); % a decreases linearly from 2 to 0
    %         %a=2*(1-l^power_for_a*(1/Max_iter^power_for_a)); % a decreases linearly from 2 to 0
    %     end
    
    %    disp(a)
    
    % Update the Position of search agents including omegas
    for i=1:nb_search_agent %%% for each wolf
        
        %%%%%%% Discrete parameters
        
        if isempty(Discrete_Indices)==0
            %%% 1) Leader selection for the discrete parameters
            [Leader_Term_Discrete_Case,Nb_Alpha_Chosen,Nb_Beta_Chosen,Nb_Delta_Chosen]=...
                Compute_Leader_Term_Discrete_Case...
                (Positions_Indices(i,Discrete_Indices),Alpha_pos_Indices(Discrete_Indices),...
                Beta_pos_Indices(Discrete_Indices),Delta_pos_Indices(Discrete_Indices),...
                Random1_pos_Indices(Discrete_Indices),Random2_pos_Indices(Discrete_Indices),...
                Nb_Alpha_Chosen,Nb_Beta_Chosen,Nb_Delta_Chosen,a,Nb_Leaders_GWO);
            
            %Leader_Selection_and_Agent_Displacement
            
            %%% 2) Displacement towards leader
            
            
            
            [Positions_Indices(i,Discrete_Indices),Positions_Values(i,Discrete_Indices)]=...
                Displacement_Towards_Leader_Discrete_Case...
                (Positions_Indices(i,Discrete_Indices),Positions_Values(i,Discrete_Indices),Parameter_Set_Search_Sets_Discrete,...
                Leader_Term_Discrete_Case,a);
            %
            %         Positions_Values
            %
            %         pause on
            %         pause
            %
            
            % Alpha_pos_Values
            %  pause
        end
        
        if isempty(Continuous_Indices)==0
            %%%%%%% Continuous parameters
            for j=Continuous_Indices
                %%% 1) Leader selection for the continuous parameters
                %
                %         disp('Continuous value')
                %         pause on
                %         pause
                %
                [X1,X2,X3,X4,X5]=...
                    Compute_Leader_Term_Continous_Case...
                    (Positions_Values(i,j),Alpha_pos_Values(j),Beta_pos_Values(j),Delta_pos_Values(j),Random1_pos_Values(j),Random2_pos_Values(j),a);
                
                
                
                %%% 2) Displacement towards leader
                Positions_Values_i_j=...
                    Displacement_Towards_Leader_Continuous_Case(X1,X2,X3,X4,X5,Nb_Leaders_GWO);
                Positions_Values(i,j)=Positions_Values_i_j;
                
                
            end
        end
        %
        %         Positions_Values
        %
        %         pause on
        %         pause
        
        
        %           %for j=Continuous_Indices %%% for each parameter
        %
        %
        %         Leader_Selection_and_Agent_Displacement
        %
        %           end
        
        %%% Update leader
        if l==Max_iter-1
            Flag4ub=Positions_Values(i,:)>ub;
            Flag4lb=Positions_Values(i,:)<lb;
            Positions_Values(i,:)=(Positions_Values(i,:).*(~(Flag4ub+Flag4lb)))+ub.*Flag4ub+lb.*Flag4lb;
            
            fitness=Run_Objective_Function(Name_Objective_Function,Positions_Values(i,:));%,Parameter_Set_imgs
%             if fitness<=Alpha_score
%                 Alpha_score=fitness; % Update alpha
%                 Alpha_pos_Values=Positions_Values(i,:);
%                 Alpha_pos_Indices=Positions_Indices(i,:);
%             end
            Update_Leaders
        end
        
        
    end
    %%% 2.4 ms
    
    
    % Positions_Indices
    % Positions_Values
    %
    % pause on
    % pause
    
    %       l
    %
    %         Alpha_score
    %
    %         pause
    
    
    l=l+1;
    
    %     disp('after iteration l')
    %      Positions_Indices
    %     Positions_Values
    %     l
    %
    %     pause on
    %     pause
    %
    
    %      Alpha_score
    %      Alpha_pos_Values
    
    Set_Nb_Alpha_Chosen(l)=Nb_Alpha_Chosen;
    Set_Nb_Beta_Chosen(l)=Nb_Beta_Chosen;
    Set_Nb_Delta_Chosen(l)=Nb_Delta_Chosen;
    
    %     pause on
    %     pause
    %
    
    Set_a(l)=a;
    
    Convergence_curve(l)=Alpha_score;
    
    %     a
    
    %Display_Plot
    
    %l
    
    %
    %
%             Alpha_score
%     Alpha_pos_Values
%     Beta_pos_Values
%     Delta_pos_Values
%     
%     Positions_Values
% 
% Alpha_pos_Values,
% Beta_pos_Values,
% Delta_pos_Values

    Plot_Objective_Function_And_Search_Agents(Optimizer,X,Y,Z,conj(lb'),conj(ub'),l,Positions_Values,Alpha_pos_Values,Beta_pos_Values,Delta_pos_Values) 

    
    %     
    
%     
%     pause on
%     pause
    
    %
    
    %
    %     if (Alpha_score==min_FRR)%||(a<=a_Stop_Value)
    %
    %         %
    %         Final_Iter_Index=l;
    %         % Convergence_curve=Convergence_curve(1:Final_Iter_Index);
    %
    % %         disp('final iteration value')
    % %         disp(Final_Iter_Index)
    % %         %break
    % %
    % %         pause on
    % %         pause
    %
    %     end
    
    %     end
    %end
    
    
    Alpha_pos=Alpha_pos_Values;

    %      Alpha_score
    %     pause on
    %     pause
    % Alpha_pos_Values
    % l
    
    
    % disp('Number of times each is chosen, at iteration 1 and at last iteration')
    % disp('Alpha')
    % Set_Nb_Alpha_Chosen(1),Set_Nb_Alpha_Chosen(end)
    % disp('Beta')
    % Set_Nb_Beta_Chosen(1),Set_Nb_Beta_Chosen(end)
    % disp('Delta')
    % Set_Nb_Delta_Chosen(1),Set_Nb_Delta_Chosen(end)
    % disp('First and last values of a')
    % Set_a(1),Set_a(end)
    % pause on
    % pause
    
    
    %     end
    
end

%         time_Displacement =toc    %%% 7 e-4
%
%         pause on
%         pause
%

%end


end
