% Modified Grey Wolf Optimizer
%%% inspired by:

%%% the a parameter is polynomial
%___________________________________________________________________%
%  Grey Wold Optimizer (GWO) source codes version 1.0               %
%                                                                   %
%  Developed in MATLAB R2011b(7.13)                                 %
%                                                                   %
%  Author and programmer: Seyedali Mirjalili                        %
%                                                                   %
%         e-Mail: ali.mirjalili@gmail.com                           %
%                 seyedali.mirjalili@griffithuni.edu.au             %
%                                                                   %
%       Homepage: http://www.alimirjalili.com                       %
%                                                                   %
%   Main paper: S. Mirjalili, S. M. Mirjalili, A. Lewis             %
%               Grey Wolf Optimizer, Advances in Engineering        %
%               Software , in press,                                %
%               DOI: 10.1016/j.advengsoft.2013.12.007               %
%                                                                   %
%___________________________________________________________________%


%%%% 

%Parameter_Set_imgs,
function [Alpha_score,Alpha_pos,Convergence_curve,Final_Iter_Index]=Modified_GWO_Safe...
    (nb_search_agent,Nb_Leaders_GWO,...
    it_max,power_for_a,...
    Parameter_Set_Lower_Bounds,Parameter_Set_Upper_Bounds,Parameter_Set_Alpha_Initial_Position,...
    min_FRR,dim,Name_Objective_Function,Parameter_Set_imgs,a_Stop_Value)




SearchAgents_no=nb_search_agent;
Max_iter=it_max;
lb=conj(Parameter_Set_Lower_Bounds(:)');
ub=conj(Parameter_Set_Upper_Bounds(:)');



% initialize alpha, beta, and delta_pos
Alpha_pos=zeros(1,dim);
Alpha_score=inf; %change this to -inf for maximization problems

Beta_pos=zeros(1,dim);
Beta_score=inf; %change this to -inf for maximization problems

Delta_pos=zeros(1,dim);
Delta_score=inf; %change this to -inf for maximization problems

%Initialize the positions of search agents
Positions=initialization(SearchAgents_no,dim,ub,lb,Parameter_Set_Alpha_Initial_Position);

Convergence_curve=zeros(1,Max_iter);

l=0;% Loop counter

Final_Iter_Index=Max_iter;% default value
% Main loop

while l<Max_iter
    %l
    for i=1:size(Positions,1)  
        
       % Return back the search agents that go beyond the boundaries of the search space
        Flag4ub=Positions(i,:)>ub;
        Flag4lb=Positions(i,:)<lb;
        Positions(i,:)=(Positions(i,:).*(~(Flag4ub+Flag4lb)))+ub.*Flag4ub+lb.*Flag4lb;               
        
        % Calculate objective function for each search agent
        %Positions(i,:)
                
        fitness=Run_Objective_Function(Name_Objective_Function,Positions(i,:),Parameter_Set_imgs);
        %%%%% Pour les fonctions Matlab Name_Objective_Function est:
        %%% Run_Matlab_Criterion_With_One_Parameter_Set
        
        
        
        % Update Alpha, Beta, and Delta
        if fitness<Alpha_score 
            Alpha_score=fitness; % Update alpha
            Alpha_pos=Positions(i,:);
            %Alpha_pos
        end
        
        
        if fitness>Alpha_score && fitness<Beta_score 
            Beta_score=fitness; % Update beta
            Beta_pos=Positions(i,:);
        end
        
        if fitness>Alpha_score && fitness>Beta_score && fitness<Delta_score 
            Delta_score=fitness; % Update delta
            Delta_pos=Positions(i,:);
        end
    end
    
    
    a=2*(1-l^power_for_a*(1/Max_iter^power_for_a)); % a decreases linearly from 2 to 0

    
    % Update the Position of search agents including omegas
    for i=1:size(Positions,1)
        for j=1:size(Positions,2)     
                       
            r1=rand(); % r1 is a random number in [0,1]
            r2=rand(); % r2 is a random number in [0,1]
            
            A1=2*a*r1-a; % Equation (3.3)
            C1=2*r2; % Equation (3.4)
            
            D_alpha=abs(C1*Alpha_pos(j)-Positions(i,j)); % Equation (3.5)-part 1
            X1=Alpha_pos(j)-A1*D_alpha; % Equation (3.6)-part 1
                       
            r1=rand();
            r2=rand();
            
            A2=2*a*r1-a; % Equation (3.3)
            C2=2*r2; % Equation (3.4)
            
            D_beta=abs(C2*Beta_pos(j)-Positions(i,j)); % Equation (3.5)-part 2
            X2=Beta_pos(j)-A2*D_beta; % Equation (3.6)-part 2       
            
            r1=rand();
            r2=rand(); 
            
            A3=2*a*r1-a; % Equation (3.3)
            C3=2*r2; % Equation (3.4)
            
            D_delta=abs(C3*Delta_pos(j)-Positions(i,j)); % Equation (3.5)-part 3
            X3=Delta_pos(j)-A3*D_delta; % Equation (3.5)-part 3             
            
            
            Positions(i,j)=(X1+X2+X3)/3;% Equation (3.7)
            
        end
    end
    l=l+1;    
    %Alpha_score
    Convergence_curve(l)=Alpha_score;
%     a 

Positions_Values=Positions;
Alpha_pos_Values=Alpha_pos;
Beta_pos_Values=Beta_pos;
Delta_pos_Values=Delta_pos;


%Display_Plot



%%%%% Name_Objective_Function_Core est dfinie dans:
        %%% Run_Matlab_Criterion_With_One_Parameter_Set
 
%Define_Name_Objective_Function_Core

% figure(2)
% Complete_Plot_Function(Function_name,Positions,[Beta_pos;Delta_pos],Alpha_pos)
% 
% hold off

      
%     if (Alpha_score<=min_FRR)||(a<=a_Stop_Value)
% 
%         Final_Iter_Index=l;
% Convergence_curve=Convergence_curve(1:Final_Iter_Index);
%         break
%     end
    
end


end




