%___________________________________________________________________%
%  Grey Wold Optimizer (GWO) source codes version 1.0               %
%                                                                   %
%  Developed in MATLAB R2011b(7.13)                                 %
%                                                                   %
%  Author and programmer: Seyedali Mirjalili                        %
%                                                                   %
%         e-Mail: ali.mirjalili@gmail.com                           %
%                 seyedali.mirjalili@griffithuni.edu.au             %
%                                                                   %
%       Homepage: http://www.alimirjalili.com                       %
%                                                                   %
%   Main paper: S. Mirjalili, S. M. Mirjalili, A. Lewis             %
%               Grey Wolf Optimizer, Advances in Engineering        %
%               Software , in press,                                %
%               DOI: 10.1016/j.advengsoft.2013.12.007               %
%                                                                   %
%___________________________________________________________________%

% You can simply define your cost in a seperate file and load its handle to fobj 
% The initial parameters that you need are:
%__________________________________________
% fobj = @YourCostFunction
% dim = number of your variables
% Max_iteration = maximum number of generations
% SearchAgents_no = number of search agents
% lb=[lb1,lb2,...,lbn] where lbn is the lower bound of variable n
% ub=[ub1,ub2,...,ubn] where ubn is the upper bound of variable n
% If all the variables have equal lower bound you can just
% define lb and ub as two single number numbers

% To run GWO: [Best_score,Best_pos,GWO_cg_curve]=GWO(SearchAgents_no,Max_iteration,lb,ub,dim,fobj)
%__________________________________________

clear all 
close all
clc

Name_Of_The_Folder_As_A_String='.\Results\';
%Optimization_Methods={'amixedGWO','GWO','PSO','PSOGSA','MVO'};% 'GWO','PSO','PSOGSA'
%Optimization_Methods={'amixedGWO','MVO','GWO','PSO','PSOGSA'};%,'MVO'
Optimization_Methods={'amixedGWO','GWO'};% 



%%% F17 to F23: fixed dimension
%%% It is not recommended to plot them in a 2D space because it does not
%%% represent the actual values of the function, which do not exist in 2D.

%for ind=8:10
 ind=1;
 Function_name=['F',num2str(ind)]; % Name of the test function that can be from F1 to F23 (Table 1,2,3 in the paper)
%Function_name='Halong';%F1  % NDparabola %Extreme_Slopes  % Halong Mountains2D




%%% Fonctions qui montrent bien l'intrt du a Mixed GWO:
%%% 


% Load details of the selected benchmark function

%Function_name='ackley';
[lb,ub,dim,fobj]=Get_Functions_details(Function_name);

SearchAgents_no=10; % Number of search agents

Max_iteration=16; % Maximum numbef of iterations
Nb_Methods=length(Optimization_Methods);

Best_score_All_Methods=zeros(1,Nb_Methods);
Best_pos_All_Methods=zeros(dim,Nb_Methods);
Convergence_Curve_All_Methods=zeros(Max_iteration,Nb_Methods);

% [PSO_Best_score,PSO_Best_pos,PSO_cg_curve]=PSO(SearchAgents_no,Max_iteration,lb,ub,dim,fobj); % run PSO to compare to results
 % [GWO_Best_score,GWO_Best_pos,GWO_cg_curve]=GWO(SearchAgents_no,Max_iteration,lb,ub,dim,fobj);
%  [Best_score,Best_pos,cg_curve]=MVO(SearchAgents_no,Max_iteration,lb,ub,dim,fobj);
% [Mixed_GWO_Best_score,Mixed_GWO_Best_pos,Mixed_GWO_cg_curve]=amixedGWO(SearchAgents_no,Max_iteration,lb,ub,dim,fobj);

figure(11)
%figure('Position',[500 500 500 300])
%Draw search space
%subplot(1,2,1);
func_plot(Function_name);
title('Parameter space')
xlabel('x_2');
ylabel('x_1');
zlabel([Function_name,'( x_1 , x_2 )'])
axis tight
grid on
box on
% pause
%print -depsc -r650 ObjFunc
%close(11)

for Index_Method=1:Nb_Methods
    
    Optimizer=str2func(Optimization_Methods{1,Index_Method});    
    disp('We are running: ')
    disp(Optimizer)

    [Best_score,Best_pos,Convergence_Curve]=feval(Optimizer,SearchAgents_no,Max_iteration,lb,ub,dim,fobj,Optimizer);
    pause
    
Best_score_All_Methods(1,Index_Method)=Best_score;
Best_pos_All_Methods(:,Index_Method)=Best_pos(:);
Convergence_Curve_All_Methods(:,Index_Method)=Convergence_Curve;
end

% 
% figure(12)
% %figure('Position',[500 500 500 300])
% %Draw search space
% %subplot(1,2,1);
% func_plot(Function_name);
% title('Parameter space')
% xlabel('x_2');
% ylabel('x_1');
% zlabel([Function_name,'( x_1 , x_2 )'])

Indices_Iterations=[1:Max_iteration];
Ylabel='criterion value';    
Option_Log=1;
    
Display_Signals_with_Legend_Generic(Convergence_Curve_All_Methods,...
        9,['CCrvNb_Iter_',num2str(Max_iteration),'_Cost_Function_',Function_name],[Name_Of_The_Folder_As_A_String,'Convergence_curves\'],...
        Indices_Iterations,Optimization_Methods,'iteration',Ylabel,Option_Log)
    

% GWO_cg_curve
% PSO_cg_curve

% display(['The best solution obtained by GWO is : ', num2str(GWO_Best_pos)]);
% display(['The best optimal value of the objective function found by GWO is : ', num2str(GWO_Best_score)]);
% 
% display(['The best solution obtained by PSO is : ', num2str(PSO_Best_pos)]);
% display(['The best optimal value of the objective function found by PSO is : ', num2str(PSO_Best_score)]);

for Index_Method=1:Nb_Methods

Best_pos=Best_pos_All_Methods(:,Index_Method);
Best_pos=conj(Best_pos(:)');

Best_score=Best_score_All_Methods(:,Index_Method);
Best_score=conj(Best_score(:)');

disp(['Method ',Optimization_Methods{1,Index_Method},': ']);

disp(['Best solution: ',num2str(Best_pos)]);
Best_score=Best_score_All_Methods(:,Index_Method);
disp(['The best optimal value: ',num2str(Best_score)]);
disp('------------------')
end

%end

